@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: downloader-view.c,v 1.21 2003/07/14 14:22:37 chpe Exp $
+ *  $Id: downloader-view.c,v 1.23 2003/08/08 17:37:14 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -394,19 +394,19 @@ downloader_view_update_details (DownloaderViewPrivate *priv,
 	}
 	else if (details->size_total > 0)
 	{
-		sprintf (buffer, _("%d of %d KB"),
+		sprintf (buffer, _("%d of %d kB"),
 			 details->size_done,
 			 details->size_total);
 	}
 	else
 	{
-		sprintf (buffer, _("%d KB"),
+		sprintf (buffer, _("%d kB"),
 			 details->size_done);
 	}
 
 	if (details->speed > 0)
 	{
-		sprintf (buffer, _("%s at %.1f KB/s"), buffer, details->speed);
+		sprintf (buffer, _("%s at %.1f kB/s"), buffer, details->speed);
 	}
 	gtk_label_set_text (GTK_LABEL (priv->details_status),
 			    buffer);
@@ -517,7 +517,7 @@ downloader_view_set_download_info (DownloaderViewPrivate *priv,
 	}
 	else if (details->size_total > 0)
 	{
-		sprintf (buffer, "%d KB", details->size_total);
+		sprintf (buffer, "%d kB", details->size_total);
 	}
 	else
 	{
@@ -588,6 +588,7 @@ downloader_view_add_download (DownloaderView *dv,
 	GtkTreeIter iter;
 	DownloadDetails *details;
 	GtkTreeSelection *selection;
+	GtkTreePath *path;
 
 	details = g_new0 (DownloadDetails, 1);
 	details->filename = g_strdup (filename);
@@ -609,10 +610,10 @@ downloader_view_add_download (DownloaderView *dv,
 	gtk_list_store_append (GTK_LIST_STORE (dv->priv->model),
 			       &iter);
 
+	path =  gtk_tree_model_get_path (GTK_TREE_MODEL (dv->priv->model), &iter);
 	details->ref = gtk_tree_row_reference_new
-		(GTK_TREE_MODEL (dv->priv->model),
-		 gtk_tree_model_get_path
-		 (GTK_TREE_MODEL (dv->priv->model), &iter));
+				(GTK_TREE_MODEL (dv->priv->model),path);
+	gtk_tree_path_free (path);
 
 	gtk_list_store_set (GTK_LIST_STORE (dv->priv->model),
 			    &iter,
