@@ -14,6 +14,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: print-dialog.c,v 1.8 2003/08/08 17:31:04 chpe Exp $
  */
 
 #include "print-dialog.h"
@@ -21,22 +23,21 @@
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkstock.h>
 
-#define CONF_PRINT_BOTTOM_MARGIN "/apps/epiphany/print/bottom_margin"
-#define CONF_PRINT_TOP_MARGIN "/apps/epiphany/print/top_margin"
-#define CONF_PRINT_LEFT_MARGIN "/apps/epiphany/print/left_margin"
-#define CONF_PRINT_RIGHT_MARGIN "/apps/epiphany/print/right_margin"
-#define CONF_PRINT_PAGE_TITLE "/apps/epiphany/print/page_title_toggle"
-#define CONF_PRINT_PAGE_URL "/apps/epiphany/print/page_url_toggle"
-#define CONF_PRINT_DATE "/apps/epiphany/print/date_toggle"
-#define CONF_PRINT_PAGE_NUMBERS "/apps/epiphany/print/page_numbers_toggle"
-#define CONF_PRINT_PRINTER "/apps/epiphany/print/printer"
-#define CONF_PRINT_FILE "/apps/epiphany/print/file"
-#define CONF_PRINT_PRINTON "/apps/epiphany/print/printon"
-#define CONF_PRINT_PAPER "/apps/epiphany/print/paper"
-#define CONF_PRINT_ALL_PAGES "/apps/epiphany/print/all_pages"
-#define CONF_PRINT_START_FROM_LAST "/apps/epiphany/print/start_from_last"
-#define CONF_PRINT_COLOR "/apps/epiphany/print/color"
-#define CONF_PRINT_ORIENTATION "/apps/epiphany/print/orientation"
+#define CONF_PRINT_BOTTOM_MARGIN "/apps/epiphany/dialogs/print_bottom_margin"
+#define CONF_PRINT_TOP_MARGIN "/apps/epiphany/dialogs/print_top_margin"
+#define CONF_PRINT_LEFT_MARGIN "/apps/epiphany/dialogs/print_left_margin"
+#define CONF_PRINT_RIGHT_MARGIN "/apps/epiphany/dialogs/print_right_margin"
+#define CONF_PRINT_PAGE_TITLE "/apps/epiphany/dialogs/print_page_title"
+#define CONF_PRINT_PAGE_URL "/apps/epiphany/dialogs/print_page_url"
+#define CONF_PRINT_DATE "/apps/epiphany/dialogs/print_date"
+#define CONF_PRINT_PAGE_NUMBERS "/apps/epiphany/dialogs/print_page_numbers"
+#define CONF_PRINT_PRINTER "/apps/epiphany/dialogs/print_printer"
+#define CONF_PRINT_FILE "/apps/epiphany/dialogs/print_file"
+#define CONF_PRINT_PRINTON "/apps/epiphany/dialogs/print_on"
+#define CONF_PRINT_PAPER "/apps/epiphany/dialogs/print_paper"
+#define CONF_PRINT_ALL_PAGES "/apps/epiphany/dialogs/print_all_pages"
+#define CONF_PRINT_COLOR "/apps/epiphany/dialogs/print_color"
+#define CONF_PRINT_ORIENTATION "/apps/epiphany/dialogs/print_orientation"
 
 static void print_dialog_class_init (PrintDialogClass *klass);
 static void print_dialog_init (PrintDialog *dialog);
@@ -96,7 +97,7 @@ EphyDialogProperty properties [] =
 	{ PRINTON_PROP, "printer_radiobutton", CONF_PRINT_PRINTON, PT_NORMAL, NULL },
 	{ PRINTER_PROP, "printer_entry", CONF_PRINT_PRINTER, PT_NORMAL, NULL },
 	{ FILE_PROP, "file_entry", CONF_PRINT_FILE, PT_NORMAL, NULL },
-	{ PAPER_PROP,"letter_radiobutton", CONF_PRINT_PAPER, PT_NORMAL, NULL },
+	{ PAPER_PROP,"A4_radiobutton", CONF_PRINT_PAPER, PT_NORMAL, NULL },
 	{ TOP_PROP, "top_spinbutton", CONF_PRINT_TOP_MARGIN, PT_NORMAL, NULL },
         { BOTTOM_PROP, "bottom_spinbutton", CONF_PRINT_BOTTOM_MARGIN, PT_NORMAL, NULL },
 	{ LEFT_PROP,"left_spinbutton", CONF_PRINT_LEFT_MARGIN, PT_NORMAL, NULL },
@@ -114,6 +115,13 @@ EphyDialogProperty properties [] =
 	{ -1, NULL, NULL }
 };
 
+static const
+char *paper_format_enum [] =
+{
+	"A4", "Letter", "Legal", "Executive"
+};
+static guint n_paper_format_enum = G_N_ELEMENTS (paper_format_enum);
+
 static guint print_dialog_signals[LAST_SIGNAL] = { 0 };
 
 GType
@@ -180,6 +188,9 @@ print_dialog_init (PrintDialog *dialog)
 				 "print.glade", "print_dialog");
 
 	dialog->priv->window = ephy_dialog_get_control (EPHY_DIALOG(dialog), WINDOW_PROP);
+
+	ephy_dialog_add_enum (EPHY_DIALOG (dialog), PAPER_PROP,
+			      n_paper_format_enum, paper_format_enum);
 	
 	icon = gtk_widget_render_icon (dialog->priv->window, 
 						      GTK_STOCK_PRINT,
@@ -213,7 +224,7 @@ print_dialog_new (EphyEmbed *embed,
 	PrintDialog *dialog;
 
 	dialog = PRINT_DIALOG (g_object_new (PRINT_DIALOG_TYPE,
-				     "EphyEmbed", embed,
+				     "embed", embed,
 				     NULL));
 
 	if (!embed) dialog->only_collect_info = TRUE;
@@ -230,7 +241,7 @@ print_dialog_new_with_parent (GtkWidget *window,
 	PrintDialog *dialog;
 
 	dialog = PRINT_DIALOG (g_object_new (PRINT_DIALOG_TYPE,
-				     "EphyEmbed", embed,
+				     "embed", embed,
 				     "ParentWindow", window,
 				     NULL));
 
@@ -245,9 +256,12 @@ print_free_info (EmbedPrintInfo *info)
 {
 	g_free (info->printer);
 	g_free (info->file);
+	g_free (info->paper);
 	g_free (info->header_left_string);
+	g_free (info->header_center_string);
 	g_free (info->header_right_string);
 	g_free (info->footer_left_string);
+	g_free (info->footer_center_string);
 	g_free (info->footer_right_string);
 	g_free (info);
 }
@@ -255,7 +269,7 @@ print_free_info (EmbedPrintInfo *info)
 static EmbedPrintInfo *
 print_get_info (EphyDialog *dialog)
 {
-        EmbedPrintInfo *info;
+	EmbedPrintInfo *info;
 	GValue print_to_file = {0, };
 	GValue printer = {0, };
 	GValue file = {0, };
@@ -274,67 +288,84 @@ print_get_info (EphyDialog *dialog)
 	GValue date = {0, };
 	GValue page_numbers = {0, };
 
-        info = g_new0 (EmbedPrintInfo, 1);
+	info = g_new0 (EmbedPrintInfo, 1);
 
 	ephy_dialog_get_value (dialog, PRINTON_PROP, &print_to_file);
-        info->print_to_file = g_value_get_int (&print_to_file);
+	info->print_to_file = g_value_get_int (&print_to_file);
+	g_value_unset (&print_to_file);
 
 	ephy_dialog_get_value (dialog, PRINTER_PROP, &printer);
 	info->printer = g_strdup (g_value_get_string (&printer));
+	g_value_unset (&printer);
 
 	ephy_dialog_get_value (dialog, FILE_PROP, &file);
-        info->file = g_strdup (g_value_get_string (&file));
+	info->file = g_strdup (g_value_get_string (&file));
+	g_value_unset (&file);
 
 	ephy_dialog_get_value (dialog, BOTTOM_PROP, &bottom_margin);
-        info->bottom_margin = g_value_get_float (&bottom_margin);
+	info->bottom_margin = g_value_get_float (&bottom_margin);
+	g_value_unset (&bottom_margin);
 
 	ephy_dialog_get_value (dialog, LEFT_PROP, &left_margin);
-        info->left_margin = g_value_get_float (&left_margin);
+	info->left_margin = g_value_get_float (&left_margin);
+	g_value_unset (&left_margin);
 
 	ephy_dialog_get_value (dialog, TOP_PROP, &top_margin);
-        info->top_margin = g_value_get_float (&top_margin);
+	info->top_margin = g_value_get_float (&top_margin);
+	g_value_unset (&top_margin);
 
 	ephy_dialog_get_value (dialog, RIGHT_PROP, &right_margin);
-        info->right_margin = g_value_get_float (&right_margin);
+	info->right_margin = g_value_get_float (&right_margin);
+	g_value_unset (&right_margin);
 
 	ephy_dialog_get_value (dialog, FROM_PROP, &from_page);
-        info->from_page = g_value_get_float (&from_page);
+	info->from_page = g_value_get_float (&from_page);
+	g_value_unset (&from_page);
 
 	ephy_dialog_get_value (dialog, TO_PROP, &to_page);
-        info->to_page = g_value_get_float (&to_page);
+	info->to_page = g_value_get_float (&to_page);
+	g_value_unset (&to_page);
 
 	ephy_dialog_get_value (dialog, PAPER_PROP, &paper);
-        info->paper = g_value_get_int (&paper);
+	info->paper = g_strdup (paper_format_enum[g_value_get_int (&paper)]);
+	g_value_unset (&paper);
 
 	ephy_dialog_get_value (dialog, ALL_PAGES_PROP, &pages);
-        info->pages = g_value_get_int (&pages);
+	info->pages = g_value_get_int (&pages);
+	g_value_unset (&pages);
 
 	ephy_dialog_get_value (dialog, COLOR_PROP, &print_color);
-        info->print_color = !g_value_get_int (&print_color);
+	info->print_color = !g_value_get_int (&print_color);
+	g_value_unset (&print_color);
 
 	ephy_dialog_get_value (dialog, ORIENTATION_PROP, &orientation);
-        info->orientation = g_value_get_int (&orientation);
+	info->orientation = g_value_get_int (&orientation);
+	g_value_unset (&orientation);
 
-        info->frame_type = 0;
+	info->frame_type = 0;
 
 	ephy_dialog_get_value (dialog, PAGE_TITLE_PROP, &page_title);
-        info->header_left_string = g_value_get_boolean (&page_title) ?
+	info->header_left_string = g_value_get_boolean (&page_title) ?
 				   g_strdup ("&T") : g_strdup ("");
+	g_value_unset (&page_title);
 
 	ephy_dialog_get_value (dialog, PAGE_URL_PROP, &page_url);
-        info->header_right_string = g_value_get_boolean (&page_url) ?
+	info->header_right_string = g_value_get_boolean (&page_url) ?
 				    g_strdup ("&U") : g_strdup ("");
+	g_value_unset (&page_url);
 
 	ephy_dialog_get_value (dialog, PAGE_NUMBERS_PROP, &page_numbers);
-        info->footer_left_string = g_value_get_boolean (&page_numbers) ?
+	info->footer_left_string = g_value_get_boolean (&page_numbers) ?
 				   g_strdup ("&PT") : g_strdup ("");
+	g_value_unset (&page_numbers);
 
 	ephy_dialog_get_value (dialog, DATE_PROP, &date);
-        info->footer_right_string = g_value_get_boolean (&date) ?
+	info->footer_right_string = g_value_get_boolean (&date) ?
 				    g_strdup ("&D") : g_strdup ("");
+	g_value_unset (&date);
 
-        info->header_center_string = g_strdup("");
-        info->footer_center_string = g_strdup("");
+	info->header_center_string = g_strdup("");
+	info->footer_center_string = g_strdup("");
 
 	return info;
 }
@@ -355,6 +386,7 @@ print_dialog_print (EphyDialog *dialog)
 	{
 		embed = ephy_embed_dialog_get_embed
 			(EPHY_EMBED_DIALOG(dialog));
+		g_return_if_fail (embed != NULL);
 
 		info->preview = FALSE;
 		ephy_embed_print (embed, info);
@@ -380,6 +412,7 @@ print_dialog_preview (EphyDialog *dialog)
 	{
 		embed = ephy_embed_dialog_get_embed
 			(EPHY_EMBED_DIALOG(dialog));
+		g_return_if_fail (embed != NULL);
 
 		info->preview = TRUE;
 		ephy_embed_print (embed, info);
