@@ -127,6 +127,9 @@ extern void ephy_marshal_VOID__POINTER_POINTER (GClosure     *closure,
                                                 gpointer      invocation_hint,
                                                 gpointer      marshal_data);
 
+/* VOID:FLOAT (./ephy-marshal.list:18) */
+#define ephy_marshal_VOID__FLOAT	g_cclosure_marshal_VOID__FLOAT
+
 G_END_DECLS
 
 #endif /* __ephy_marshal_MARSHAL_H__ */
